import { select } from 'd3';
import topojson from 'topojson-client';
import { map } from './map';

const worldAtlasURL =
  'https://unpkg.com/visionscarto-world-atlas@0.1.0/world/110m.json';

export const main = (container, { state, setState }) => {
  const width = window.innerWidth;
  const height = window.innerHeight;

  const svg = select(container)
    .selectAll('svg')
    .data([null])
    .join('svg')
    .attr('width', width)
    .attr('height', height);

  // state.data could be:
  // * undefined
  // * 'LOADING'
  // * An array of objects
  const { data, rotation } = state;

  if (data && data !== 'LOADING' && rotation) {
    svg.call(map, {
      data,
      rotation,
    });
  }

  if (data === undefined) {
    setState((state) => ({
      ...state,
      data: 'LOADING',
    }));
    fetch(worldAtlasURL)
      .then((response) => response.json())
      .then((topoJSONData) => {
        const data = topojson.feature(
          topoJSONData,
          'countries',
        );
        setState((state) => ({
          ...state,
          data,
        }));
      });
  }

  if (rotation === undefined) {
    setState((state) => ({
      ...state,
      rotation: [0, -20, 0],
    }));
    return;
  }

  const enableAnimation = true;
  let animationFrameRequestId;
  if (enableAnimation) {
    animationFrameRequestId = requestAnimationFrame(() => {
      animationFrameRequestId = undefined;
      setState((state) => ({
        ...state,
        rotation: [rotation[0] - 1 / 10, -20, 0],
      }));
    });
  }

  // This does not work well with hot reloading.
  // if (rotation === undefined) {
  //   setState((state) => ({
  //     ...state,
  //     rotation: [0, -20, 0],
  //   }));
  //   const animate = () => {
  //     setState((state) => ({
  //       ...state,
  //       rotation: [state.rotation[0] - 9 / 10, -20, 0],
  //     }));
  //     requestAnimationFrame(animate);
  //   };
  //   animate();
  // }

  const cleanup = () => {
    // If this is defined, there is a pending request
    // for an animation frame that should be canceled.
    if (animationFrameRequestId !== undefined) {
      cancelAnimationFrame(animationFrameRequestId);
    }
  };
  return cleanup;
};
